/* Copyright 2005, 2007 Ricardo Baratto */
/* Copyright 2005, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
/* 
 * zsound_test_enc.c: test program for sound encoding library
 *
 * Authors: 
 * Ricardo A. Baratto - Network Computing Lab - Columbia University
 */

/* reads ogg from stdin, decodes it, and writes wav to stdout */

/* set tabstop = 8 */
#include "thincCommon.h"
#include "thinc_zsound.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#define PAGE_SIZE	4096

#define error(...)	fprintf(stderr, ## __VA_ARGS__)

#if 0
#define TZ_TEST_DO_DEBUG
#endif

#ifdef TZ_TEST_DO_DEBUG
#define debug(...)	fprintf(stderr, ## __VA_ARGS__)
#else
#define debug(...)
#endif

static int test_read(void *handle, void *data, unsigned int len);

int main(int argc, char **argv)
{
	struct thinc_zsound *zs;
	int err;
	char *buffer;

	debug("hi there, welcome to the test program...\n");

	zs = thinc_zsound_dec_init(T_ZSOUND_VORBIS);
	buffer = malloc(PAGE_SIZE);
	if(!zs || !buffer) {
		error("no init 4 u\n");
		return EXIT_FAILURE;
	}

	debug("i'm all initialized and almost ready to go, let me prepare "
		"something first tho\n");

	thinc_zsound_dec_setcb(zs, test_read, NULL);

	err = thinc_zsound_dec_prepare(zs);
	if(err == T_ERROR) {
		error("can't prepare\n");
		return EXIT_FAILURE;
	}

	debug("k, now we ready, let's have some fun!\n");

	/* decode data */
	while(1) {
		err = thinc_zsound_dec_decode(zs, buffer, PAGE_SIZE);
		if(err == T_ERROR) {
			error("can't decode\n");
			return EXIT_FAILURE;
		} else if(err == T_ZSOUND_DONE) {
			debug("\nk, we're done\n");
			break;
		}
		fwrite(buffer, err, 1, stdout);
	}

	debug("seems we're out of data, let's finish things up\n");
	err = thinc_zsound_dec_finish(zs);
	if(err == T_ERROR) {
		error("can't finish\n");
	}

	thinc_zsound_dec_end(zs);

	debug("and that was that... hope you didn't die\n");
	return EXIT_SUCCESS;
}

static int test_read(void *handle, void *data, unsigned int len)
{
	static long total = 0;
	int err;
	
	err = fread(data, 1, len, stdin);
	total+= err;
	debug("read %d bytes, %ld total\n", err, total);
	return err;
}
