/* Copyright 2004, 2005, 2007 Ricardo Baratto */
/* Copyright 2004, 2005, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * test program for resize library
 *
 * Authors: 
 * Ricardo A. Baratto - Network Computing Lab - Columbia University
 */
/* set tabstop = 8 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <netinet/in.h>

#include <assert.h>

#include <sys/time.h>
#include <time.h>

#include "thincCommon.h"
#include "thinc_resize.h"

/* 
 * input files should have header:
 * [ width (4) | height (4) | bpp (4) ]
 * [ data (width * height * bpp ]
 */

#define gettime_ms(t) { \
        struct timeval tv;              \
        gettimeofday(&tv, NULL);        \
        t = (tv.tv_sec*1000) + (tv.tv_usec/1000); \
}

int do_per_channel(int argc, char **argv)
{
	unsigned int inlen, outlen, kernel_type, param, inw, inh, outw, outh;
	unsigned int bpp, d;
	int infd, outfd, err;
	unsigned char *in, *out;
	unsigned long t1, t2;

	/* read params */
	outw = atoi(argv[3]);
	outh = atoi(argv[4]);
	kernel_type = atoi(argv[5]);
	param = atoi(argv[6]);

	/* open input file */
	infd = open(argv[1], O_RDONLY);
	if(infd == -1) {
		fprintf(stderr, "%s: Couldn't open input file %s: %s\n",
			argv[0], argv[1], strerror(errno));
		return 1;
	}

	outfd = open(argv[2], O_RDWR|O_CREAT, 
		S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if(outfd == -1) {
		fprintf(stderr, "%s: Couldn't open output file %s: %s\n",
			argv[0], argv[2], strerror(errno));
		return 1;
	}

	read(infd, &inw, 4);
	read(infd, &inh, 4);
	read(infd, &bpp, 4);
	read(infd, &d, 4);	/* pad */
	inw = ntohl(inw);
	inh = ntohl(inh);
	bpp = ntohl(bpp);
	inlen = inw*inh*bpp;
	outlen = outw*outh*bpp;

	printf("resizing image from %u*%u to %u*%u. bpp is %u\n", inw, inh,
		outw, outh, bpp);
	printf("input size is %u, output size is %u\n", inlen, outlen);

	/* map file */
	in = malloc(inlen);
	if(!in) {
		fprintf(stderr, "%s: could not allocate memory for input "
			"data\n", argv[0]);
		return 1;
	}
	read(infd, in, inlen);
	out = malloc(outlen);
	if(!out) {
		fprintf(stderr, "%s: no memory for output data: %s\n",
			argv[0], strerror(errno));
		return 1;
	}

	/* resize */
	printf("resizing...\n");
	gettime_ms(t1);
#if 0
	err = thinc_resize32_old(in, out, inw, inh, outw, outh, inw*bpp,
		outw*bpp, kernel_type, param);
#else
	err = thinc_resize16(in, out, inw, inh, outw, outh, inw*bpp, 
		outw*bpp);
#endif
	gettime_ms(t2);
	if(err == T_ERROR) {
		printf("\n%s: Could not resize input file\n", argv[0]);
		close(infd);
		close(outfd);
		return 1;
	}
	printf("done. took us %lums to do it\n", t2-t1);

	outw = htonl(outw);
	outh = htonl(outh);
	bpp = htonl(bpp);
	write(outfd, &outw, 4);
	write(outfd, &outh, 4);
	write(outfd, &bpp, 4);
	write(outfd, &d, 4);
	write(outfd, out, outlen);
	free(out);

	/* done */
	close(infd);
	close(outfd);

	printf("c-ya!\n");
	return 0;
}

#if 0
int do_rgba(int argc, char **argv)
{
	unsigned int inlen, outlen, kernel_type, param, inw, inh, outw, outh;
	unsigned int byteinlen, byteoutlen, d;
	unsigned int bpp;
	int infd, outfd, err;
	unsigned long int *in, *out;
	unsigned long t1, t2;

	/* read params */
	outw = atoi(argv[3]);
	outh = atoi(argv[4]);
	kernel_type = atoi(argv[5]);
	param = atoi(argv[6]);

	/* open input file */
	infd = open(argv[1], O_RDONLY);
	if(infd == -1) {
		fprintf(stderr, "%s: Couldn't open input file %s: %s\n",
			argv[0], argv[1], strerror(errno));
		return 1;
	}

	outfd = open(argv[2], O_RDWR|O_CREAT, 
		S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if(outfd == -1) {
		fprintf(stderr, "%s: Couldn't open output file %s: %s\n",
			argv[0], argv[2], strerror(errno));
		return 1;
	}

	read(infd, &inw, 4);
	read(infd, &inh, 4);
	read(infd, &bpp, 4);
	read(infd, &d, 4);	/* pad */
	inw = ntohl(inw);
	inh = ntohl(inh);
	bpp = ntohl(bpp);	/* this one assumes 32bpp */
	assert(bpp == sizeof(unsigned long int));

	inlen = inw*inh;
	byteinlen = inlen*sizeof(unsigned long int);
	outlen = outw*outh;
	byteoutlen = outlen*sizeof(unsigned long int);

	printf("resizing image from %u*%u to %u*%u. bpp is %u\n", inw, inh,
		outw, outh, bpp);
	printf("input size is %u, output size is %u\n", inlen, outlen);

	/* map file */
	in = malloc(byteinlen);
	if(!in) {
		fprintf(stderr, "%s: could not allocate memory for input "
			"data\n", argv[0]);
		return 1;
	}
	read(infd, in, byteinlen);
	out = malloc(byteoutlen);
	if(!out) {
		fprintf(stderr, "%s: no memory for output data: %s\n",
			argv[0], strerror(errno));
		return 1;
	}

	/* resize */
	printf("resizing...\n");
	gettime_ms(t1);
	err = thinc_resize_long(in, out, inlen, outlen, kernel_type, param);
	gettime_ms(t2);
	if(err == T_ERROR) {
		printf("\n%s: Could not resize input file\n", argv[0]);
		munmap(in, inlen);
		close(infd);
		close(outfd);
		return 1;
	}
	printf("done. took us %lums to do it\n", t2-t1);

	outw = htonl(outw);
	outh = htonl(outh);
	bpp = htonl(bpp);
	write(outfd, &outw, 4);
	write(outfd, &outh, 4);
	write(outfd, &bpp, 4);
	write(outfd, &d, 4);
	write(outfd, out, byteoutlen);
	free(out);

	/* done */
	close(infd);
	close(outfd);

	printf("c-ya!\n");
	return 0;
}
#endif

int main(int argc, char **argv)
{
	if(argc != 7) {
		fprintf(stderr, "usage: %s IN OUT width height kernel_type "
			"param\n If your kernel_type doesn't need param, "
			"pass 0\n", argv[0]);
		return 1;
	}

#if 1
	return do_per_channel(argc, argv);
#else
	return do_rgba(argc, argv);
#endif
}
