/**************************************************************************
 *
 * Copyright 2002-2006 Columbia University
 * All Rights Reserved
 *
 **************************************************************************/
/*
 * thincProtoSound.h
 *
 * Client/server sound protocol definition for THINC
 *
 * Authors: 
 * Leo Kim (lnk2101@cs.columbia.edu)
 * Ricardo A Baratto
 * Network Computing Lab - Columbia University
 */

/* set tabstop = 8 */
#ifndef _THINC_PROTO_SOUND_H_
#define _THINC_PROTO_SOUND_H_

/* Message type */
#define T_SND_DATA			70
#define T_SND_PLAYBACK_OPEN		71
#define T_SND_PLAYBACK_END		73
#define T_SND_CAPTURE_OPEN		74
#define T_SND_CAPTURE_END		75
#define T_SND_VOLUME_DOWN		76
#define T_SND_VOLUME_UP			77

/* Data structures */

/* sent by tsd to a new client. for now we just use it when using UDP, and
 * it contains the UDP port tsd will use to transmit and receive audio
 * data. If the port is 0, then UDP will not be used. We don't add a 
 * client->tsd message, instead the client should just reuse whatever random
 * port the TCP connection gets assigned. yup, pretty obscure 
 * (and unportable?), but am lazy today. */
typedef struct thinc_sndInit_s {
	int port	:16;
	char pad[2];
} thinc_sndInit;

#define T_SND_INIT_SIZE		2

/* Tells the client to open its sound card and defines the format of the sound
 * stream. May be used for playback or capture. */
typedef struct thinc_sndOpen_s {
	int id		:8;
	int bits        :8;
	int rate        :16;
	int channels    :8;
	int endian      :8;
	char pad[2];
} thinc_sndOpen;

#define T_SND_OPEN_SIZE   	6

/* carries some audio data. May be used for playback or capture */
typedef struct thinc_sndData_s {
	int size 		:32;
	int timestamp		:32;
	int id			:8;
	char pad[3];
} thinc_sndData;

#define T_SND_DATA_SIZE   	9

/* Teardown the passed audio stream (playback or capture) */
typedef struct thinc_sndClose_s {
	/* Tell which ID to close.*/
	int id			:8;
	char pad[3];
} thinc_sndClose;

#define T_SND_CLOSE_SIZE   	1

/** for UDP **/
/* how much data to send at a time */
#define T_SND_UDP_PKT_SIZE	(1U<<12)	/* 4K */

/* UDP data. Has T_SND_DATA type basically wraps around thinc_header and 
 * thinc_sndData so that
 * we can read it all in one swoop. however, we don't use thinc_sndData
 * explicitly to minimize obscurity (since if we use it we cannot use
 * T_SND_*_SIZE macros due to its extra padding) */
typedef struct thinc_sndUDPData_s {
	thinc_header		header;
	int size		:32;	/* in case we're sending less */
	int timestamp		:32;
	int id			:8;
	char pad[3];			/* need to keep things aligned */
	char data[T_SND_UDP_PKT_SIZE];
} thinc_sndUDPData;

/* we cannot use T_SND_*_SIZE macros because thinc_sndData has some extra
 * padding that it's not accounted for in T_SND_DATA_SIZE (by design).
 * at the moment of writing, this size is 4112. please keep up to date
 * if you change something */

#define T_SND_UDP_DATA_HSIZE	THINC_HEADERSIZE + 12 /* thinc_sndData */
#define T_SND_UDP_DATA_SIZE T_SND_UDP_DATA_HSIZE + T_SND_UDP_PKT_SIZE

#endif	/* _THINC_PROTO_SOUND_H */
