#!/usr/bin/perl
#
# Hack to parse the thinc client's output and draw an image with nitfy colored 
# rectangles representing the different messages that the client received.
# Try it, it's cool!
#
# Released to the public domain. You may use this
# code any way you wish, private, educational, or commercial.  It's free.
#
# Authors:
# Ricardo A. Baratto - Network Computing Lab - Columbia University
#
# usage:
# client-graph.pl logfile imagefile
#
# set tabstop=4 #

use strict;
use GD;
use Getopt::Std;

use vars qw(
	$opt_o
	$opt_r
	$opt_f
	$opt_p
	$opt_g
	$opt_b
	$opt_B
	$opt_c
);

# -o => dont redraw repeated rectangles (o -> just Once)
# -r => dont draw raw
# -f => dont draw fills
# -p => dont draw pixmaps
# -g => dont draw glyphs
# -b => dont draw bilevel
# -B => don't draw black border
# -c => don't draw copies
getopts('orfpgbBc');

main(@ARGV);

sub main {
	my ($infile, $outfile, $im, $white, $red, $green, $blue, $purple);
	my ($black, $yellow, $gray, %rects, $count, $bluish);
	my ($x, $y, $xpos, $ypos, $w, $h);
	my ($raw, $fill, $pix, $glyph, $blev, $copy, $size, $png_data);
	# borders
	my ($braw);

	$infile = shift || die("usage: $0 infile outfile $!\n");
	$outfile = shift || die("usage: $0 infile outfile $!\n");

	open(INFILE, $infile) || die "can't open input file $infile: $!\n";

	# find out the size of the fb. in a very hackish way
	my ($w, $h, $bpp);
	$w = 0;
	$h = 0;
	while(<INFILE>) {
		chomp;

		s/\((\d){2}:(\d){2}:(\d){2}\)\s//;

		if(/creating fb/) {
			s/.*creating fb, size is //;

			($w, $h, $bpp) = split(x);
			last;
		}
	}

	# went through the file and couldn't figure out fb size. not good
	if($w == 0 or $h == 0) {
		die "i don't know the size of the framebuffer dude!\n";
	}

	print "framebuffer is $w x $h\n";
	# new image
	$im = new GD::Image($w,$h);

	# allocate some colors
	# raw -> red
	# sfill -> blue
	# pfill -> green
	# glyph -> purple
	# bilevel -> bluish
	# warning -> yellow
	$white = $im->colorAllocate(255,255,255);
	$black = $im->colorAllocate(0,0,0);
	$gray = $im->colorAllocate(140,140,140);
	$red = $im->colorAllocate(200,0,0);
	$green = $im->colorAllocate(0,200,0);
	$blue = $im->colorAllocate(0,0,200);
	$purple = $im->colorAllocate(200,0,200);
	$yellow = $im->colorAllocate(200,200,0);
	$bluish = $im->colorAllocate(0,200,200);

	$im->rectangle(0,0,$w,$h,$black);

	%rects = ();

	$count = 0;

	while(<INFILE>) {
		chomp;

		# strip off useless text
		# TODO We could keep the time and report it if user wants to
		s/^\(II\)\s//;
		s/^\(EE\)\s//;
		s/^\(WW\)\s//;
        
		s/\((\d){2}:(\d){2}:(\d){2}\)\s//;

		if(!$opt_c) {
		if(/handleCopy/) {
			if(/parameters/) {
				my ($x1, $y1, $x2, $y2);
				s/[a-zA-Z_: ]//g;
				print "GOT COPY $_\n";
				($x1,$y1,$x2,$y2,$w,$h) = split(/,/);
				$im->setThickness(3);
				$im->line($x1,$y1,$x2,$y2,$black);
				$im->filledRectangle($x1,$y1,$x1+4,$y1+4,$blue);
				$im->filledRectangle($x2,$y2,$x2+4,$y2+4,$blue);
				$im->line($x1+$w,$y1+$h,$x2+$w,$y2+$h,$black);
				$im->filledRectangle($x1+$w,$y1+$h,$x1+$w+4,$y1+$h+4,$blue);
				$im->filledRectangle($x2+$w,$y2+$h,$x2+$w+4,$y2+$h+4,$blue);
				$im->setThickness(1);
			}
		}
		}

		if(!$opt_r) {
		if(/handleRUpdate/) {
			if(/parameters/) {
				s/[a-zA-Z_: ]//g;
				s/,//;
				if(! $rects{$_}) {
					$rects{$_} = 1;
					$raw = $red;
				} else {
					$rects{$_}++;

					if(!$opt_o) {
						if($rects{$_} == 2) {
							$raw = $yellow;
						} else {
							$raw = $gray;
						}
					} else {
						next;
					}
					print "got repeated rectangle $_.";
					print "have seen it $rects{$_} times\n";
				}
				($x,$y,$w,$h) = split(/,/);

				if($w < 10) {
					$ypos = $y + $h + 10;
				} else {
					$ypos = $y + ($h/2);
				}
				if($h < 10) {
					$xpos = $x + $w + 10;
				} else {
					$xpos = $x + ($w/2);
				}
				$size = $w * $h * 4;
				$im->filledRectangle($x,$y,$x+$w,$y+$h, $raw);
				if(!$opt_B) {
					$braw = $black;
				} else {
					$braw = $raw;
				}
				$im->rectangle($x,$y,$x+$w,$y+$h, $braw);
#				$im->string(gdGiantFont,$xpos, $ypos, $size, $black);

				++$count;
			}
		}
		}

		if(!$opt_f) {
		if(/handleSFill/) {
			if(/rectangle /) {
				s/[a-zA-Z_ ]//g;
				s/://;
				s/[0-9]://;

				if(! $rects{$_}) {
					$rects{$_} = 1;
					$fill = $blue;
				} else {
					$rects{$_}++;
					if(!$opt_o) {
						if($rects{$_} == 2) {
							$fill = $yellow;
						} else {
							$fill = $gray;
						}
					} else {
						next;
					}

					print "got repeated rectangle $_.";
					print "have seen it $rects{$_} times\n";
				}
				($x,$y,$w,$h) = split(/,/);

				if($w < 10) {
					$ypos = $y + $h + 10;
				} else {
					$ypos = $y + ($h/2);
				}
				if($h < 10) {
					$xpos = $x + $w + 10;
				} else {
					$xpos = $x + ($w/2);
				}

				$size = $w * $h * 4;

				$im->filledRectangle($x,$y,$x+$w,$y+$h,	$fill);
				$im->rectangle($x,$y,$x+$w,$y+$h,$black);
#				$im->string(gdSmallFont,$xpos, $ypos, $size, $black);

				++$count;
			}
		}
		}

		if(!$opt_p) {
		if(/handlePFill/) {
			if(/rectangle/) {
				s/[a-zA-Z_:]//g;

				if(! $rects{$_}) {
					$rects{$_} = 1;
					$pix = $green;
				} else {
					$rects{$_}++;
					if(!$opt_o) {
						if($rects{$_} == 2) {
							$pix = $yellow;
						} else {
							$pix = $gray;
						}
					} else {
						next;
					}
					print "got repeated rectangle $_.";
					print "have seen it $rects{$_} times\n";
				}
				($x,$y,$w,$h) = split(/,/);

				if($w < 10) {
					$ypos = $y + $h + 10;
				} else {
					$ypos = $y + ($h/2);
				}
				if($h < 10) {
					$xpos = $x + $w + 10;
				} else {
					$xpos = $x + ($w/2);
				}

				$size = $w * $h * 4;

				$im->filledRectangle($x,$y,$x+$w,$y+$h, $pix);
				$im->rectangle($x,$y,$x+$w,$y+$h,$black); 
#				$im->string(gdSmallFont,$xpos, $ypos, $size, $black);

				++$count;
			}
		}
		}

		if(!$opt_g) {
		if(/handleGlyph/) {
			if(/rectangle/) {
				s/[a-zA-Z_:]//g;

				if(! $rects{$_}) {
					$rects{$_} = 1;
					$glyph = $purple;
				} else {
					$rects{$_}++;
					if(!$opt_o) {
						if($rects{$_} == 2) {
							$glyph = $yellow;
						} else {
							$glyph = $gray;
						}
					} else {
						next;
					}
					print "got repeated rectangle $_.";
					print "have seen it $rects{$_} times\n";
				}
				($x,$y,$w,$h) = split(/,/);

				if($w < 10) {
					$ypos = $y + $h + 10;
				} else {
					$ypos = $y + ($h/2);
				}
				if($h < 10) {
					$xpos = $x + $w + 10;
				} else {
					$xpos = $x + ($w/2);
				}

				$size = $w * $h * 4;

				$im->filledRectangle($x,$y,$x+$w,$y+$h,	$glyph);
				$im->rectangle($x,$y,$x+$w,$y+$h, $black);
#				$im->string(gdSmallFont,$xpos, $ypos, $size, $black);

				++$count;
			}
		}
		}

		if(!$opt_b) {
		if(/handleBilevel/) {
			if(/rectangle/) {
				s/[a-zA-Z_:]//g;

				if(! $rects{$_}) {
					$rects{$_} = 1;
					$blev = $bluish;
				} else {
					$rects{$_}++;
					if(!$opt_o) {
						if($rects{$_} == 2) {
							$blev = $yellow;
						} else {
							$blev = $gray;
						}
					} else {
						next;
					}
					print "got repeated rectangle $_.";
					print "have seen it $rects{$_} times\n";
				}
				($x,$y,$w,$h) = split(/,/);

				if($w < 10) {
					$ypos = $y + $h + 10;
				} else {
					$ypos = $y + ($h/2);
				}
				if($h < 10) {
					$xpos = $x + $w + 10;
				} else {
					$xpos = $x + ($w/2);
				}

				$size = $w * $h * 4;

				$im->filledRectangle($x,$y,$x+$w,$y+$h,	$blev);
				$im->rectangle($x,$y,$x+$w,$y+$h,$black);

#				$im->string(gdSmallFont,$xpos, $ypos, $size, $black);

				++$count;
			}
		}
		}


	}

	print ("draw $count rectangles\n");

	close(INFILE);

	# save the image
	open(OUTFILE, ">$outfile") || 
		die ("can't open output file $outfile: $!\n");

	binmode OUTFILE;

	$png_data = $im->png;

	print OUTFILE $png_data;

	close(OUTFILE);

	print "done.\n";
}
