/* 
 * Routines for THINC protocol dissection.
 *
 * Authors: 
 * Ricardo A. Baratto - Network Computing Lab - Columbia University
 *
 * At moment of writing, it's unclear to me if this code works at all, nor
 * how complete it is.
 *
 * Released to the public domain. You may use this
 * code any way you wish, private, educational, or commercial.  It's free.
 */

/* set tabstop = 4 */

/* Reference:
 * to access tvbuff_t:
   guint8 tvb_get_guint8(tvbuff_t*, gint offset)
   
   guint16 tvb_get_ntohs(tvbuff_t*, gint offset)
   guint32 tvb_get_ntohl(tvbuff_t*, gint offset)
   guint32 tvb_get_ntoh24(tvbuff_t*, gint offset)

   guint8* tvb_memcpy(tvbuff_t*, guint8* target, gint offset, gint length)
   guint8* tvb_memdup(tvbuff_t*, gint off, gint length)

 * columns:
 * COL_PROTOCOL and COL_INFO
 * to check if column should be displayed call
 * check_col() => TRUE/FALSE
 *
 * col_set_str() => argument must be string constant or static const array
 * for other cases use 
 * col_add_str()/col_add_fstr() => fstr() -> printf-like
 * col_clear()
 * col_append_str()/col_append_fstr() => string is appended to current value
 * for the column
 *
 * protocol tree
 * -> created when when packet selected in packet-list pane
 * -> for every protocol or field that a dissector wants to register, 
 * a variable of type int needs to be used to keep track of the protocol.
 * ID: establish parent-child rels between protos and fields.
 */


#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <epan/packet.h>

/* protocol and registered fields */
static int proto_thinc = -1;

/* all msgs */
static int hf_thinc_type = -1;
static int hf_thinc_flags = -1;
static int hf_thinc_length = -1;

/* server -> client */
static int hf_thinc_sstart_byte_order = -1;
static int hf_thinc_sstart_version1 = -1;
static int hf_thinc_sstart_version2 = -1;
static int hf_thinc_sstart_version3 = -1;
static int hf_thinc_sack_ack = -1;
static int hf_thinc_fbinit_width = -1;
static int hf_thinc_fbinit_height = -1;
static int hf_thinc_fbinit_depth = -1;
static int hf_thinc_fbinit_bpp = -1;
static int hf_thinc_fbinit_cursor_size = -1;
static int hf_thinc_fbinit_cursor_fg = -1;
static int hf_thinc_fbinit_cursor_bg = -1;
static int hf_thinc_fbinit_cursor_width = -1;
static int hf_thinc_fbinit_cursor_height = -1;
static int hf_thinc_fbinit_cursor_x = -1;
static int hf_thinc_fbinit_cursor_y = -1;
static int hf_thinc_fbinit_cursor_xhot = -1;
static int hf_thinc_fbinit_cursor_yhot = -1;
static int hf_thinc_rupdate_x = -1;
static int hf_thinc_rupdate_y = -1;
static int hf_thinc_rupdate_width = -1;
static int hf_thinc_rupdate_height = -1;
static int hf_thinc_copy_sx = -1;
static int hf_thinc_copy_sy = -1;
static int hf_thinc_copy_dx = -1;
static int hf_thinc_copy_dy = -1;
static int hf_thinc_copy_width = -1;
static int hf_thinc_copy_height = -1;
static int hf_thinc_sfill_pixel = -1;
static int hf_thinc_sfill_numrects = -1;
static int hf_thinc_pfill_x = -1;
static int hf_thinc_pfill_y = -1;
static int hf_thinc_pfill_width = -1;
static int hf_thinc_pfill_height = -1;
static int hf_thinc_pfill_numrects = -1;
static int hf_thinc_glyph_x = -1;
static int hf_thinc_glyph_y = -1;
static int hf_thinc_glyph_width = -1;
static int hf_thinc_glyph_height = -1;
static int hf_thinc_glyph_color = -1;
static int hf_thinc_glyph_numrects = -1;
static int hf_thinc_bilevel_x = -1;
static int hf_thinc_bilevel_y = -1;
static int hf_thinc_bilevel_width = -1;
static int hf_thinc_bilevel_height = -1;
static int hf_thinc_bilevel_fg = -1;
static int hf_thinc_bilevel_bg = -1;
static int hf_thinc_bilevel_numrects = -1;
static int hf_thinc_rect_x = -1;
static int hf_thinc_rect_y = -1;
static int hf_thinc_rect_width = -1;
static int hf_thinc_rect_height = -1;
static int hf_thinc_zdata_zsize = -1;
static int hf_thinc_zdata_usize = -1;
static int hf_thinc_cursor_change_xhot = -1;
static int hf_thinc_cursor_change_yhot = -1;
static int hf_thinc_cursor_showhide_sh = -1;
static int hf_thinc_cursor_move_x = -1;
static int hf_thinc_cursor_move_y = -1;
static int hf_thinc_cursor_color_fg = -1;
static int hf_thinc_cursor_color_bg = -1;

/* client -> server */
/* XXX: We can't really recognize these because they may have the same
 * opcode as server -> client :*( 
 * Maybe we can write a thinc-server and a thinc-client dissector? */
#if 0
static int hf_thinc_cstart_version = -1;
static int hf_thinc_cack_ack = -1;
static int hf_thinc_evmotion_x = -1;
static int hf_thinc_evmotion_y = -1;
static int hf_thinc_evbutton_mask = -1;
static int hf_thinc_evbutton_mask_button = -1;
static int hf_thinc_evbutton_mask_updown = -1;
static int hf_thinc_evkeyb_key = -1;
static int hf_thinc_evkeyb_down = -1;
#endif

/* subtree pointers TODO TODO TODO */
static gint ett_thinc = -1;

/* this is the real deal */
static void dissect_thinc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	const char *func = __func__;
	
	/* structs needed to add protocol subtree and manage it */
	proto_item *ti;
	proto_tree *thinc_tree;

	/* make entries in protocol column and info column on summary
	 * display */
	if(check_col(pinfo->cinfo, COL_PROTOCOL))
		col_set_str(pinfo->cinfo, COL_PROTOCOL, "THINC");

	/* This field shows up as the "Info" column in the display. It
	 * should summarize what's in the packet, so that a user looking at 
	 * the list of packets can tell what type of packet it is.
	 * 
	 * If you are setting it to a constant string, use "col_set_str()", as
	 * it's more efficient than the other "col_set_XXX()" calls.
	 *
	 * If you're setting it to a string you've constructed, or will be
	 * appending to the column later, use "col_add_str()".
	 * 
	 * "col_add_fstr()" can be used instead of "col_add_str()"; it takes
	 * "printf()"-like arguments.  Don't use "col_add_fstr()" with a 
	 * format string of "%s" - just use "col_add_str()" or 
	 * "col_set_str()", as it's more efficient than "col_add_fstr()".
	 * 
	 * If you will be fetching any data from the packet before filling in
	 * the Info column, clear that column first, in case the calls to 
	 * fetch data from the packet throw an exception because they're 
	 * fetching data past the end of the packet, so that the Info 
	 * column doesn't have data left over from the previous dissector; do
	 */

	/* good thing, we'll probably need to get data from the packet to
	 * know what kind of request this is */
	if (check_col(pinfo->cinfo, COL_INFO)) 
		col_clear(pinfo->cinfo, COL_INFO);

	/* TODO TODO TODO 
	 * 
	 * Need to figure out how to know what kind of request it is :)
	 *
	 * TODO TODO TODO 
	 */
	if (check_col(pinfo->cinfo, COL_INFO)) 
		col_set_str(pinfo->cinfo, COL_INFO, "XXX Request");
	
	/* In the interest of speed, if "tree" is NULL, avoid building a
	 * protocol tree and adding stuff to it if possible.  Note,
	 * however, that you must call subdissectors regardless of whether
	 * "tree" is NULL or not. */
	if (tree) {
		/* NOTE: The offset and length values in the call to
		 * "proto_tree_add_item()" define what data bytes to highlight in 
		 * the hex display window when the line in the protocol tree 
		 * display corresponding to that item is selected.
		 * 
		 * Supplying a length of -1 is the way to highlight all data from 
		 * the offset to the end of the packet. */
		
		/* create display subtree for the protocol */
		ti = proto_tree_add_item(tree, proto_thinc, tvb, 0, -1, FALSE);
		
		thinc_tree = proto_item_add_subtree(ti, ett_thinc);
		
		/* add an item to the subtree, see section 1.6 for more information */

		/* TODO TODO TODO TODO */
		proto_tree_add_item(thinc_tree,
			hf_thinc_FIELDABBREV, tvb, offset, len, FALSE);
		
		/* Continue adding tree items to process the packet here */
	}
	
	/* If this protocol has a sub-dissector call it here, see section 1.8 */
}


/* Register the protocol with Ethereal */
void proto_register_thinc(void)
{
	/* Setup list of header fields */

#if 0
	typedef struct hf_register_info {
		int			*p_id;	
		header_field_info	hfinfo;
	} hf_register_info;

	struct _header_field_info {
		/* ---------- set by dissector --------- */
		char				*name;
		char				*abbrev;
		enum ftenum			type;
		int				display;	/* for integers only, so far. Base */
		const void			*strings;	/* val_string or true_false_string */
		guint32				bitmask;
		char				*blurb;		/* Brief description of field. */
		
		/* ---------- set by proto routines --------- */
		int				id;		/* Field ID */
		int				parent;		/* parent protocol */
		int				bitshift;	/* bits to shift */
		
		header_field_info		*same_name_next; /* Link to next hfinfo 
													with same abbrev*/
		header_field_info		*same_name_prev; /* Link to previous 
													hfinfo with same abbrev*/
	};
#endif
	/* TODO: 
	 * - do value_string array for thinc.type. Problem is client and
	 *   server have same type fields for their messages. how to 
	 *   distinguish :?
	 */
#define thinc_hf_8 (hf, name, shortname) \
		{ &hf, \
			{ name, shortname, \
			FT_UINT8, BASE_DEC, NULL, 0, "", HFILL } \
		}
#define thinc_hf_16 (hf, name, shortname) \
		{ &hf, \
			{ name, shortname, \
			FT_UINT16, BASE_DEC, NULL, 0, "", HFILL } \
		}
#define thinc_hf_32 (hf, name, shortname) \
		{ &hf, \
			{ name, shortname, \
			FT_UINT32, BASE_DEC, NULL, 0, "", HFILL } \
		}

	static hf_register_info hf[] = {
		/* header */
		thinc_hf_8(hf_thinc_type, "Message Type", "thinc.type"),
		thinc_hf_8(hf_thinc_flags, "Message Flags", "thinc.flags"),
		thinc_hf_16(hf_thinc_length, "Message Length", "thinc.length"),
		/* sstart */
		thinc_hf_8(hf_thinc_sstart_byte_order,"Byte Order", 
			"thinc.sstart.byte_order"),
		thinc_hf_8(hf_thinc_sstart_version1, "Version 1", 
			"thinc.sstart.version1"),
		thinc_hf_8(hf_thinc_sstart_version2, "Version 2",
			"thinc.sstart.version2"),
		thinc_hf_8(hf_thinc_sstart_version3, "Version 3", 
			"thinc.sstart.version1"),
		/* sack */
		thinc_hf_8(hf_thinc_sack_ack, "Ack", "thinc.sack"),
		/* fb init */
		thinc_hf_16(hf_thinc_fbinit_width, "FB Width", "thinc.fb.width"),
		thinc_hf_16(hf_thinc_fbinit_height, "FB Height", "thinc.fb.height"),
		thinc_hf_8(hf_thinc_fbinit_depth, "FB Depth", "thinc.fb.depth"),
		thinc_hf_8(hf_thinc_fbinit_bpp, "FB bpp", "thinc.fb.bpp"),
		thinc_hf_16(hf_thinc_fbinit_cursor_size, "FB Cursor Size", 
			"thinc.fb.cursor.size"),
		thinc_hf_32(hf_thinc_fbinit_cursor_fg, "FB Cursor FG", 
			"thinc.fb.cursor.fg"),
		thinc_hf_32(hf_thinc_fbinit_cursor_bg, "FB Cursor BG", 
			"thinc.fb.cursor.bg"),
		thinc_hf_8(hf_thinc_fbinit_cursor_width, "FB Cursor Width", 
			"thinc.fb.cursor.width"),
		thinc_hf_8(hf_thinc_fbinit_cursor_height, "FB Cursor Height", 
			"thinc.fb.cursor.height"),
		thinc_hf_16(hf_thinc_fbinit_cursor_x, "FB Cursor X Pos", 
			"thinc.fb.cursor.xpos"),
		thinc_hf_16(hf_thinc_fbinit_cursor_y, "FB Cursor Y Pos", 
			"thinc.fb.cursor.ypos"),
		thinc_hf_8(hf_thinc_fbinit_cursor_xhot, "FB Cursor X Hot", 
			"thinc.fb.cursor.xhot"),
		thinc_hf_8(hf_thinc_fbinit_cursor_yhot, "FB Cursor Y Hot", 
			"thinc.fb.cursor.yhot"),
		/* rupdate */
		thinc_hf_16(hf_thinc_rupdate_x, "RAW X", "thinc.raw.x"),
		thinc_hf_16(hf_thinc_rupdate_y, "RAW Y", "thinc.raw.y"),
		thinc_hf_16(hf_thinc_rupdate_width, "RAW Width", "thinc.raw.width"),
		thinc_hf_16(hf_thinc_rupdate_height, "RAW Height", "thinc.raw.height"),
		/* copy */
		thinc_hf_16(hf_thinc_copy_sx, "COPY Source X", "thinc.copy.sx"),
		thinc_hf_16(hf_thinc_copy_sy, "COPY Source Y", "thinc.copy.sy"),
		thinc_hf_16(hf_thinc_copy_dx, "COPY Destination X", "thinc.copy.dx"),
		thinc_hf_16(hf_thinc_copy_dy, "COPY Destination Y", "thinc.copy.dy"),
		thinc_hf_16(hf_thinc_copy_width, "COPY Width", "thinc.copy.width"),
		thinc_hf_16(hf_thinc_copy_height, "COPY Height", "thinc.copy.height"),
		/* solid fill */
		thinc_hf_32(hf_thinc_sfill_pixel, "SFILL Color", "thinc.sfill.color"),
		thinc_hf_16(hf_thinc_sfill_numrects, "SFILL Rectangle Count", 
			"thinc.sfill.numrects"),
		/* pixmap fill */
		thinc_hf_16(hf_thinc_pfill_x, "PFILL X", "thinc.pfill.x"),
		thinc_hf_16(hf_thinc_pfill_y, "PFILL Y", "thinc.pfill.y"),
		thinc_hf_16(hf_thinc_pfill_width, "PFILL Width", "thinc.pfill.width"),
		thinc_hf_16(hf_thinc_pfill_height, "PFILL Height", 
			"thinc.pfill.height"),
		thinc_hf_16(hf_thinc_pfill_numrects, "PFILL Rectangle Count", 
			"thinc.pfill.numrects"),
		/* glyph */
		thinc_hf_16(hf_thinc_glyph_x, "GLYPH X", "thinc.glyph.x"),
		thinc_hf_16(hf_thinc_glyph_y, "GLYPH Y", "thinc.glyph.y"),
		thinc_hf_16(hf_thinc_glyph_width, "GLYPH Width", "thinc.glyph.width"),
		thinc_hf_16(hf_thinc_glyph_height, "GLYPH Height", 
			"thinc.glyph.height"),
		thinc_hf_32(hf_thinc_glyph_color, "GLYPH Color", "thinc.glyph.color"),
		thinc_hf_16(hf_thinc_glyph_numrects, "GLYPH Rectangle Count", 
			"thinc.glyph.numrects"),
		/* bilevel */
		thinc_hf_16(hf_thinc_bilevel_x, "BILEVEL X", "thinc.bilevel.x"),
		thinc_hf_16(hf_thinc_bilevel_y, "BILEVEL Y", "thinc.bilevel.y"),
		thinc_hf_16(hf_thinc_bilevel_width, "BILEVEL Width", 
			"thinc.bilevel.width"),
		thinc_hf_16(hf_thinc_bilevel_height, "BILEVEL Height", 
			"thinc.bilevel.height"),
		thinc_hf_32(hf_thinc_bilevel_fg, "BILEVEL FG", "thinc.bilevel.fg"),
		thinc_hf_32(hf_thinc_bilevel_bg, "BILEVEL BG", "thinc.bilevel.bg"),
		thinc_hf_16(hf_thinc_bilevel_numrects, "BILEVEL Rectangle Count", 
			"thinc.bilevel.numrects"),
		/* rectangle region */
		thinc_hf_16(hf_thinc_rect_x, "RECT X", "thinc.rect.x"),
		thinc_hf_16(hf_thinc_rect_y, "RECT Y", "thinc.rect.y"),
		thinc_hf_16(hf_thinc_rect_width, "RECT Width", "thinc.rect.width"),
		thinc_hf_16(hf_thinc_rect_height, "RECT Height", "thinc.rect.height"),
		/* compresse data */
		thinc_hf_32(hf_thinc_zdata_zsize, "ZDATA Compressed Size", 
			"thinc.zdata.zsize"),
		thinc_hf_32(hf_thinc_zdata_usize, "ZDATA Uncompressed Size", 
			"thinc.zdata.usize"),
		/* cursor change */
		thinc_hf_8(hf_thinc_cursor_change_xhot, "CURSOR X Hot", 
			"thinc.cursor.xhot"),
		thinc_hf_8(hf_thinc_cursor_change_yhot, "CURSOR Y Hot", 
			"thinc.cursor.yhot"),
		/* cursor showhide */
		thinc_hf_8(hf_thinc_cursor_showhide_sh, "CURSOR ShowHide", 
			"thinc.cursor.showhide"),
		/* cursor move */
		thinc_hf_16(hf_thinc_cursor_move_x, "CURSOR Move X", 
			"thinc.cursor.movex"),
		thinc_hf_16(hf_thinc_cursor_move_y, "CURSOR Move Y", 
			"thinc.cursor.movey"),
		/* cursor color */
		thinc_hf_32(hf_thinc_cursor_color_fg, "CURSOR Color FG", 
			"thinc.cursor.fg"),
		thinc_hf_32(hf_thinc_cursor_color_bg, "CURSOR Color BG",
			"thinc.cursor.bg")
	};

	/* Setup protocol subtree array */
	static gint *ett[] = {
		&ett_thinc,
	};

	/* Register the protocol name and description */
	proto_thinc = proto_register_protocol("THINC Protocol", "THINC", "thinc");

	/* TODO TODO TODO */
	
	/* Required function calls to register the header fields and subtrees 
	 * used */
	proto_register_field_array(proto_thinc, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));

	/* TODO TODO TODO */
}


/* If this dissector uses sub-dissector registration add a registration 
 * routine. This format is required because a script is used to find these 
 * routines and create the code that calls these routines.
 */
void proto_reg_handoff_thinc(void)
{
	dissector_handle_t thinc_handle;

	thinc_handle = create_dissector_handle(dissect_thinc,
	    proto_thinc);
	dissector_add("PARENT_SUBFIELD", ID_VALUE, thinc_handle);
}

