/* 
 * reads in a glyph dump file and prints it out as a binary 
 *
 * Released to the public domain. You may use this
 * code any way you wish, private, educational, or commercial.  It's free.
 *
 * Authors: 
 * Ricardo A. Baratto - Network Computing Lab - Columbia University
 */ 
/* set tabstop = 8 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <netinet/in.h>

static void dec_to_bin(unsigned char n);

#define do_read(fd, buf, size) \
	if(read(fd, buf, size) == -1) {	\
		perror("no love"); \
		return EXIT_FAILURE; \
	}

int main(int argc, char **argv)
{
	unsigned char *data, *curr;
	int w, h, size, boo;
	int dump, i, j, w2;
	int opt, index = 0, pad=1;
	struct option long_options[] = {
		{"nopad", 0, 0, 'p'},
		{0,0,0,0}
	};

	while(1) {
		opt = getopt_long(argc, argv, "p", long_options, &index);
		if(opt == -1) 
			break;
		switch(opt) {
			case 'p':
				printf("bitmap is not padded\n");
				pad = 0;
				break;
			default:
				printf("you slimy little bastard\n");
				return EXIT_FAILURE;
		}
	}

	if(optind > argc-1) {
		printf("you slimy little bastard\n");
		return EXIT_FAILURE;
	}

	dump = open(argv[optind], O_RDONLY);
	if(dump == -1) {
		perror("no love");
		return EXIT_FAILURE;
	}

	do_read(dump, &w, 4);
	w = ntohl(w);
	do_read(dump, &h, 4);
	h = ntohl(h);
	do_read(dump, &size, 4);
	size = ntohl(size);
	do_read(dump, &boo, 4);

	printf("w %d, h %d, size %d\n", w, h, size);

	data = malloc(size);
	if(!data) {
		perror("no love");
		return EXIT_FAILURE;
	}

	do_read(dump, data, size);
	close(dump);

	/* pad to 4 bytes */
	if(pad) 
		w2 = ((w + 31) & ~31) >> 3;
	else 
		w2 = ((w+7) >> 3);

	printf("w2 is %d, w is %d\n", w2, w);

	curr = data;
	for(i=0; i<h; ++i) {
		for(j=0; j<w2; ++j) {
			dec_to_bin(*curr++);
//			printf(" ");
		}
		printf("\n");
	}

	return EXIT_SUCCESS;
}

static void dec_to_bin(unsigned char n)
{
	int i=0 , b ;
#if 1	/* The X way */
	for(i=0;i<8;i++) {
#else	/* The "intuitive" way */
	for(i=7; i>=0; i--) {
#endif
		b = (((1<<i)&n)>0) ? 1 : 0 ;
		printf("%d",b);
	}
}
